// Simple Particle System
// Daniel Shiffman <http://www.shiffman.net>

// A class to describe a group of Particles
// An ArrayList is used to manage the list of Particles 

class FoodParticleSystem {

  ArrayList particles;    // An arraylist for all the particles

    Vector3D ac = new Vector3D(0.0,0.0);
  Vector3D ve = new Vector3D(0.0,0.0);
  Vector3D lo = new Vector3D(0,0);
  float nx;
  float ny;

  FoodParticleSystem() {
    particles = new ArrayList();              // Initialize the arraylist
    for (int i = 0; i < maxFood; i++) {
      lo = new Vector3D(random(width),random(height));
      particles.add(new Food(ac,ve,lo,foodMass));
    }
  }

  void run() {
    // Cycle through the ArrayList backwards b/c we are deleting
    for (int i = particles.size()-1; i >= 0; i--) {
      Food f = (Food) particles.get(i);
      updateParticle(f);
      if (f.dead() == true){    
        particles.remove(i);
      }
    }
  }

  int total(){
    return particles.size();    
  }

  void updateParticle(Food f){
    //Attraction between things
    for (int j = 0; j < particles.size(); j++) {        // For every Thing t[j]
      Food fj = (Food) particles.get(j);
      nx = fj.getX();
      ny = fj.getY();

      if (f.getX() != nx && f.getY() != ny) {     // Make sure we are not calculating gravtional pull on oneself
        Vector3D force = f.calcGravForce(fj);  // Use the function we wrote above
        f.add_force(force);                      // Add the force to the object to affect its acceleration
      }
    }

    //Add forces to contain objects within screen
    if (f.getY() < 0)
      f.add_force(new Vector3D(0,10));
    else if (f.getY() > height)
      f.add_force(new Vector3D(0,-10));
    if (f.getX() > width)
      f.add_force(new Vector3D(-10,0));
    else if (f.getX() < 0)
      f.add_force(new Vector3D(-10,0));

    //Add random motion
    f.add_force(new Vector3D(random(-1,1),random(-1,1)));    

    //Things follow mouse
    if (mousePressed) {
      Vector3D m = new Vector3D(mouseX,mouseY);
      Vector3D diff = Vector3D.sub(m,f.getLoc());
      diff.normalize();
      float factor = 1.0;  // Magnitude of Acceleration (not increasing it right now)
      diff.mult(factor);
      f.setAcc(diff);
    }
    f.run();                                  // Implement the rest of the object's functionality
  }

  float getX(int i){
    Food fj = (Food) particles.get(i);
    return fj.getX();
  } 
  float getY(int i){
    Food fj = (Food) particles.get(i);
    return fj.getY();
  } 

  Food getParticle(int i){
    Food fj = (Food) particles.get(i);   
    return fj;
  } 
  void addParticle(int x) {
    for (int i = 0; i < x; i++){
      lo = new Vector3D(random(width),random(height));
      particles.add(new Food(ac,ve,lo,random(3)));
    }
  }

  void remParticle(int i){
    if (particles.size() > 1){
      Food f = (Food) particles.get(i);
      f.die();
    }
  }  

  void delParticle(int i){
    particles.remove(i);
  }


  void addParticle(Food f) {
    particles.add(f);
  }

  // A method to test if the particle system still has particles
  boolean dead() {
    if (particles.isEmpty()) {
      return true;
    } 
    else {
      return false;
    }
  }

}



